import os
import sys
import time
import PIL.Image


# fix of broken paths for windows
if not sys.path:
    sys.path = []
path = os.getcwd()
if path not in sys.path:
    sys.path.insert(0, path)


import pystray
import tray_common


class PyStrayTray:

    def __init__(self):
        self.icon = None
        self.status_checker = tray_common.StatusChecker(self.show_balloon)
        if not pystray.Icon.HAS_NOTIFICATION:
            self.status_checker.toggle_notifications(False)
        self.icon = pystray.Icon('3DPrinterOS', icon=self.load_icon(), menu=pystray.Menu(*self.menu_items()))
        self.icon.run()

    def show_balloon(self, title, body):
        if self.icon and self.icon.HAS_NOTIFICATION:
            self.icon.notify(body, title)

    def are_notifications_enabled(self, *args, **kwargs):
        return self.status_checker.notifications

    def load_icon(self):
        return PIL.Image.open(tray_common.ICON_PATH)

    def on_toggle_notifications(self, _, item):
        # print('Item pre checked:', item.checked)
        # print('Item pre enabled:', item.enabled)
        # print('Notifications pre:', self.are_notifications_enabled())
        self.status_checker.toggle_notifications(not item.checked)
        # print('Item post checked:', item.checked)
        # print('Item post enabled:', item.enabled)
        # print('Notifications post:', self.are_notifications_enabled())
        #self.icon.items = self.menu_items()
        self.icon.update_menu()

    def on_enable_notifications(self, _, __):
        self.status_checker.toggle_notifications(True)

    def on_disable_notifications(self, _, __):
        self.status_checker.toggle_notifications(False)

    def on_quit(self):
        self.icon.stop()
        self.status_checker.stop()
        time.sleep(self.status_checker.LOOP_SLEEP_TIME / self.status_checker.LOOP_SLEEP_TIME + 0.01)
        tray_common.stop_app()
        sys.exit()

    def on_ui(self):
        tray_common.open_browser_tab()

    # def menu_items(self):
    #     menu = [pystray.MenuItem('Open UI', self.on_ui, default=True)]
    #     print("!!!HAS_NOTIFIS:", pystray.Icon.HAS_NOTIFICATION)
    #     if pystray.Icon.HAS_NOTIFICATION:
    #         if self.are_notifications_enabled():
    #             item = pystray.MenuItem('Enable notifications', action=self.on_enable_notifications)
    #         else:
    #             item = pystray.MenuItem('Disable notifications', action=self.on_disable_notifications)
    #         menu.append(item)
    #     menu.append(pystray.MenuItem('Quit', self.on_quit))
    #     return menu

    def menu_items(self):
        return (pystray.MenuItem('Open UI', self.on_ui, default=True),
                pystray.MenuItem(
                    'Toggle notifications',
                    action=self.on_toggle_notifications,
                    checked=self.are_notifications_enabled,
                    visible=pystray.Icon.HAS_NOTIFICATION),
                pystray.MenuItem('Quit', self.on_quit))

    # def menu_items(self):
    #     return (pystray.MenuItem('Open UI', self.on_ui, default=True),
    #             pystray.MenuItem(
    #                 'Enable notifications',
    #                 action=self.on_enable_notifications,
    #                 visible=pystray.Icon.HAS_NOTIFICATION),
    #             pystray.MenuItem(
    #                 'Disable notifications',
    #                 action=self.on_disable_notifications,
    #                 visible=pystray.Icon.HAS_NOTIFICATION),
    #             pystray.MenuItem('Quit', self.on_quit))


if __name__ == '__main__':
    icon = PyStrayTray()
